/**
 * @file
 * VVJ Tabs.
 *
 * Filename:     vvjt.js
 * Website:      https://www.flashwebcenter.com
 * Developer:    Alaa Haddad https://www.alaahaddad.com.
 */
((Drupal, drupalSettings, once) => {
  'use strict';

  Drupal.behaviors.VVJTabs = {
    attach: function (context, settings) {
      // Function to hide all panes
      function hideAllPanes(panes) {
        panes.forEach(pane => {
          if (pane) {
            pane.style.display = 'none';
            pane.setAttribute('aria-hidden', 'true');
          }
        });
      }

      // Function to remove active class from buttons
      function removeActiveClassFromButtons(buttons) {
        buttons.forEach(button => {
          if (button) {
            button.classList.remove('active');
            button.setAttribute('tabindex', '-1');
            button.setAttribute('aria-selected', 'false');
            button.setAttribute('aria-expanded', 'false');
          }
        });
      }

      // Function to handle tab clicks with event delegation
      function handleTabClicks(tabContainer, event) {
        const button = event.target.closest('.vvjt-button');
        if (!button) return;

        const btnId = button.id.split('-');
        const raw_id = btnId.slice(2).join('-');
        const paneId = `vvjt-pane-${raw_id}`;
        const currentPane = document.getElementById(paneId);
        const parentId = `#vvjt-${raw_id.split('-')[0]}`;
        const tabPanes = document.querySelectorAll(`${parentId} .vvjt-pane`);
        const tabButtons = document.querySelectorAll(`${parentId} .vvjt-button`);

        if (currentPane) {
          hideAllPanes(tabPanes);
          removeActiveClassFromButtons(tabButtons);
          currentPane.style.display = 'block';
          currentPane.setAttribute('aria-hidden', 'false');
          button.classList.add('active');
          button.removeAttribute('tabindex');
          button.setAttribute('aria-selected', 'true');
          button.setAttribute('aria-expanded', 'true');
          button.focus();
        } else {
          console.warn(`Pane with ID ${paneId} not found.`);
        }
      }

      // Function to handle keyboard navigation
      function handleKeyboardNavigation(tabButtons) {
        tabButtons.forEach((tab, index) => {
          tab.addEventListener('keydown', (event) => {
            const currentIndex = index;
            let newIndex;

            switch (event.key) {
              case 'ArrowRight':
                newIndex = (currentIndex + 1) % tabButtons.length;
                break;
              case 'ArrowLeft':
                newIndex = (currentIndex - 1 + tabButtons.length) % tabButtons.length;
                break;
              case 'Home':
                newIndex = 0;
                break;
              case 'End':
                newIndex = tabButtons.length - 1;
                break;
              default:
                return;
            }

            tabButtons.forEach(t => t.setAttribute('tabindex', '-1'));
            tabButtons[newIndex].setAttribute('tabindex', '0');
            tabButtons[newIndex].focus();
            event.preventDefault();
          });
        });
      }

      // Function to update aria-labels for tab buttons based on image alt text
      function updateAriaLabelsForButtons(tabButtons) {
        tabButtons.forEach(button => {
          const img = button.querySelector('img');
          if (img && img.alt) {
            button.setAttribute('aria-label', img.alt);
          } else {
            // If no image or alt, use button text as aria-label
            button.setAttribute('aria-label', button.textContent.trim() || 'Tab');
          }
        });
      }

      // Function to disable links within tab buttons
      function disableLinksInTabButtons(tabButtons) {
        tabButtons.forEach(button => {
          const links = button.querySelectorAll('a');
          links.forEach(link => {
            link.setAttribute('data-original-href', link.getAttribute('href'));
            link.removeAttribute('href');
            link.style.cursor = 'default';
            link.addEventListener('click', function(event) {
              event.preventDefault(); // Prevent link from navigating
            });
          });
        });
      }

      const tabContainers = once('vvjTabs', '.vvjt-inner', context);
      tabContainers.forEach(container => {
        const tabButtons = container.querySelectorAll('.vvjt-button');
        const tabPanes = container.querySelectorAll('.vvjt-pane');

        // Activate the first tab and pane by default
        if (tabButtons.length > 0) {
          tabButtons[0].classList.add('active');
          tabButtons[0].setAttribute('aria-selected', 'true');
          tabButtons[0].setAttribute('aria-expanded', 'true');
          tabButtons[0].setAttribute('tabindex', '0');
          if (tabPanes.length > 0) {
            tabPanes[0].style.display = 'block';
            tabPanes[0].setAttribute('aria-hidden', 'false');
          }
        }

        // Initialize tabindex for other buttons
        tabButtons.forEach((button, index) => {
          if (!button.classList.contains('active')) {
            button.setAttribute('tabindex', '-1');
          }

          // Attach click event listener
          button.addEventListener('click', handleTabClicks.bind(null, container));
        });

        // Handle keyboard navigation
        handleKeyboardNavigation(tabButtons);

        // Update aria-labels for buttons based on content
        updateAriaLabelsForButtons(tabButtons);

        // Disable links within tab buttons
        disableLinksInTabButtons(tabButtons);
      });
    }
  };
})(Drupal, drupalSettings, once);


